package com.ccwangluo.ccim;


import android.app.Activity;
import android.text.TextUtils;

import com.ccwangluo.ccim.listener.ResultCallback;
import com.ccwangluo.ccim.manage.CallBackManager;
import com.ccwangluo.ccim.manage.SocketManageer;
import com.ccwangluo.ccim.modle.IrequestData;
import com.xuhao.didi.socket.client.sdk.OkSocket;

import java.util.logging.Logger;

import ccim.protocol.Chat;
import ccim.protocol.Common;
import ccim.protocol.Message;
import ccim.protocol.MessageSpec;

public class CCClient {

    private Logger logger = Logger.getLogger(CCClient.class.getName());
    private final int version = 1;

    private volatile int id = 1;
    private SocketManageer socketManageer;
    private CallBackManager<Chat.ChatResponseMessage> callBackManager;

    private CCClient() {
    }

    private static CCClient mInstance;

    public static CCClient getInstance() {
        if (mInstance == null) {
            synchronized (CCClient.class) {
                if (mInstance == null) {
                    mInstance = new CCClient();
                }
            }
        }
        return mInstance;
    }

    public void init(Activity activity) {
        callBackManager = new CallBackManager<Chat.ChatResponseMessage>();
        socketManageer = new SocketManageer(callBackManager);
        socketManageer.initAuth();
    }


//    public void registerMessageListener()


    public void login(String username, String password, ResultCallback<String> resultCallback) {
        socketManageer.login(username, password, resultCallback);
    }

    public void connectChatServer() {
        if (TextUtils.isEmpty(socketManageer.getToken())) {
            return;
        }
        Chat.ConnectRequest builder = Chat.ConnectRequest.newBuilder().setToken(socketManageer.getToken()).build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder().setConnectReq(builder).setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasConnectRes()) {
                    logger.info("chat connect success");
                }
            }
        });
        id++;
    }

    /**
     * 获取联系人列表
     */
    public void getChatList() {
        Chat.PullContactsRequest builder = Chat.PullContactsRequest.newBuilder()
                .build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder().setPullContactsReq(builder).setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasPullContactsRes()) {
                    logger.info(String.valueOf(chatResponseMessage.getPullContactsRes()));
                }
            }
        });
        id++;
    }


    /**
     * 同步离线消息
     */
    public void getOfflineMessage() {
        Message.PullOfflineMessageRequest offlineMessageRequest = Message.PullOfflineMessageRequest.newBuilder().build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setPullOfflineMsgReq(offlineMessageRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasPullOfflineMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage.getPullOfflineMsgRes()));
                }
            }
        });
        id++;
    }

    /**
     * 搜索好友
     */
    public void searchUser(String userName) {
        Chat.SearchRequest searchRequest = Chat.SearchRequest.newBuilder().setUsername(userName).build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setSearchReq(searchRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSearchRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                }
            }
        });
        id++;
    }

    /**
     * 添加好友
     */
    public void addContact(Long userId) {
        Chat.AddContactRequest addContactRequest = Chat.AddContactRequest.newBuilder().setUserId(userId).build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setAddContactReq(addContactRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasAddContactRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                }
            }
        });
        id++;
    }

    /**
     * 联系人申请列表
     */
    public void getContactRequests() {
        Chat.ContactRequestsRequest contactRequestsRequest = Chat.ContactRequestsRequest.newBuilder().build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setContactRequestsReq(contactRequestsRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasContactRequestsRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                }
            }
        });
        id++;
    }

    /**
     * 验证联系人申请
     */
    public void checkContactReq(Long reqUserId,boolean accept) {
        Chat.CheckContactReqRequest checkContactReqRequest = Chat.CheckContactReqRequest.newBuilder()
                .setReqUserId(reqUserId)
                .setAccept(accept)
                .build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setCheckContactReqReq(checkContactReqRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasCheckContactReqRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                }
            }
        });
        id++;
    }

    /**
     * 创建联系人群组
     */
    public void createContactGroup(String name) {
        Chat.CreateContactGroupRequest createContactGroupRequest = Chat.CreateContactGroupRequest.newBuilder()
                .setName(name)
                .build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setCreateContactGroupReq(createContactGroupRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasCreateContactGroupRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                }
            }
        });
        id++;
    }


    /**
     * 移动联系人所在的群组
     */
    public void moveContactGroup(Long groupId,Long userId) {
        Chat.MoveContactGroupRequest moveContactGroupRequest = Chat.MoveContactGroupRequest.newBuilder()
                .setContactGroupId(groupId)
                .setContactUserId(userId)
                .build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setMoveContactGroupReq(moveContactGroupRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasMoveContactGroupRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                }
            }
        });
        id++;
    }

    /**
     * 删除联系人
     */
    public void deleteContact(Long userId) {
        Chat.DeleteContactRequest deleteContactRequest = Chat.DeleteContactRequest.newBuilder()
                .setContactUserId(userId)
                .build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setDeleteContactReq(deleteContactRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasDeleteContactRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                }
            }
        });
        id++;
    }

    /**
     * 拉黑联系人
     */
    public void blockContact(Long userId,Boolean mute) {
        Chat.BlockContactRequest blockContactRequest = Chat.BlockContactRequest.newBuilder()
                .setContactUserId(userId)
                .setMute(mute)
                .build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setBlockContactReq(blockContactRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasBlockContactRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                }
            }
        });
        id++;
    }

    /**
     * 举报违规用户
     */
    public void reportIllegalUser(String title,String content,Long userId) {
        Chat.ReportIllegalUserRequest reportIllegalUserRequest = Chat.ReportIllegalUserRequest.newBuilder()
                .setContent(content)
                .setIllegalUserId(userId)
                .setTitle(title)
                .build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setReportIllegalUserReq(reportIllegalUserRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasReportIllegalUserRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                }
            }
        });
        id++;
    }

    /**
     * 发送消息
     */
    public void sendTextMessage(String content, Long reciveUserId) {
        Message.SendMessageRequest messageRequest = Message.SendMessageRequest.newBuilder()
                .setContent(content)
                .setMsgType(MessageSpec.MessageType.TEXT)
                .setReceiverUserId(reciveUserId).build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(1).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setSendMsgReq(messageRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSendMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage.getSearchRes()));
                }
            }
        });
        id++;
    }

    /**
     * 发送群组消息
     */
    public void sendGroupMessage(String content, Long groupId) {
        Message.SendGroupMessageRequest sendGroupMessageRequest = Message.SendGroupMessageRequest.newBuilder()
                .setContent(content)
                .setMsgType(MessageSpec.MessageType.TEXT)
                .setGroupId(groupId)
                .build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(1).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setSendGroupMsgReq(sendGroupMessageRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSendGroupMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                }
            }
        });
        id++;
    }

    /**
     * 发送群组消息
     */
    public void sendChannelMessage(String content, Long channelId) {
        Message.SendChannelMessageRequest sendChannelMessageRequest = Message.SendChannelMessageRequest.newBuilder()
                .setContent(content)
                .setMsgType(MessageSpec.MessageType.TEXT)
                .setChannelId(channelId)
                .build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(1).setSeq(id).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder()
                .setSendChannelMsgReq(sendChannelMessageRequest)
                .setHeader(header).build();
        OkSocket.open(socketManageer.getChatInfo())
                .send(new IrequestData(build.toByteArray()));
        callBackManager.addCallBack(id, new ResultCallback<Chat.ChatResponseMessage>() {
            @Override
            public void onSuccess(Chat.ChatResponseMessage chatResponseMessage) {
                if (chatResponseMessage.hasSendChannelMsgRes()) {
                    logger.info(String.valueOf(chatResponseMessage));
                }
            }
        });
        id++;
    }


}
