package com.ccwangluo.ccim.manage;

import com.ccwangluo.ccim.CCClient;
import com.ccwangluo.ccim.modle.IrequestData;
import com.ccwangluo.ccim.modle.MyIReaderProtocol;
import com.ccwangluo.ccim.listener.MessageListener;
import com.ccwangluo.ccim.listener.ResultCallback;
import com.google.protobuf.InvalidProtocolBufferException;
import com.xuhao.didi.core.pojo.OriginalData;
import com.xuhao.didi.socket.client.sdk.OkSocket;
import com.xuhao.didi.socket.client.sdk.client.ConnectionInfo;
import com.xuhao.didi.socket.client.sdk.client.OkSocketOptions;
import com.xuhao.didi.socket.client.sdk.client.action.SocketActionAdapter;
import com.xuhao.didi.socket.client.sdk.client.connection.IConnectionManager;

import java.util.logging.Logger;

import ccim.protocol.Auth;
import ccim.protocol.Chat;

public class SocketManageer {
    private Logger logger = Logger.getLogger(CCClient.class.getName());

    private CallBackManager<Chat.ChatResponseMessage> callBackManager;
    public SocketManageer(CallBackManager<Chat.ChatResponseMessage> callBackManager) {
        this.callBackManager = callBackManager;
    }

    private ConnectionInfo authInfo;
    private ConnectionInfo chatInfo;
    private String token;

    private ResultCallback<String> loginCallBack;

    public String getToken() {
        return token;
    }

    public void setToken(String token) {
        this.token = token;
    }

    public Logger getLogger() {
        return logger;
    }

    public void setLogger(Logger logger) {
        this.logger = logger;
    }

    public ConnectionInfo getAuthInfo() {
        return authInfo;
    }

    public void setAuthInfo(ConnectionInfo authInfo) {
        this.authInfo = authInfo;
    }

    public ConnectionInfo getChatInfo() {
        return chatInfo;
    }

    public void setChatInfo(ConnectionInfo chatInfo) {
        this.chatInfo = chatInfo;
    }

    public void initAuth() {
        //连接参数设置(IP,端口号),这也是一个连接的唯一标识,不同连接,该参数中的两个值至少有其一不一样
        this.authInfo = initServer("117.144.156.2", 9000, new MessageListener() {
            @Override
            public void onMessageListener(OriginalData data) {
                try {
                    Auth.AuthResponseMessage chatResponseMessage = Auth.AuthResponseMessage.parseFrom(data.getBodyBytes());
                    logger.info(String.valueOf(chatResponseMessage));
                    if (chatResponseMessage.getHeader().getCode() == 0) {
                        token = chatResponseMessage.getLoginRes().getToken();
                        String server = chatResponseMessage.getLoginRes().getServer();
                        initChat(server);
                        OkSocket.open(authInfo).disconnect();
                        if (loginCallBack != null){
                            loginCallBack.onSuccess(token);
                        }
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    public void login(String username, String password, ResultCallback<String> resultCallback) {
        Auth.LoginRequest.Builder builder = Auth.LoginRequest.newBuilder().setUsername("user1").setPassword("123456").setAppId(1);
        Auth.AuthRequestMessage build = Auth.AuthRequestMessage.newBuilder().setLoginReq(builder.build()).build();
        OkSocket.open(getAuthInfo())
                .send(new IrequestData(build.toByteArray()));
        this.loginCallBack = resultCallback;
    }

    private void initChat(String server) {
        //连接参数设置(IP,端口号),这也是一个连接的唯一标识,不同连接,该参数中的两个值至少有其一不一样
        String[] split = server.split(":");
        this.chatInfo  = initServer(split[0], Integer.parseInt(split[1]), new MessageListener() {
            @Override
            public void onMessageListener(OriginalData data) {
                try {
                    Chat.ChatResponseMessage chatResponseMessage = Chat.ChatResponseMessage.parseFrom(data.getBodyBytes());
                    if (chatResponseMessage.getHeader().getCode() == 0) {
                        int seq = chatResponseMessage.getHeader().getSeq();
                        if (seq > 0){
                            ResultCallback<Chat.ChatResponseMessage> callBack = callBackManager.getCallBack(seq);
                            if (callBack != null){
                                callBack.onSuccess(chatResponseMessage);
                                callBackManager.removeCallBack(seq);
                            }
                        } else { //主动接受的消息

                        }

                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    private ConnectionInfo initServer(String ip, int port, MessageListener messageListener) {
        ConnectionInfo authInfo = new ConnectionInfo(ip, port);
        //调用OkSocket,开启这次连接的通道,拿到通道Manager
        IConnectionManager manager = OkSocket.open(authInfo);
        //获得当前连接通道的参配对象
        OkSocketOptions options = manager.getOption();
        //基于当前参配对象构建一个参配建造者类
        OkSocketOptions.Builder builder = new OkSocketOptions.Builder(options);
        //修改参配设置(其他参配请参阅类文档)
        //        builder.setSinglePackageBytes(size);
        //建造一个新的参配对象并且付给通道
        manager.option(builder.build());
        //调用通道进行连接
        manager.connect();

        //设置自定义解析头
        OkSocketOptions.Builder okOptionsBuilder = new OkSocketOptions.Builder();
        okOptionsBuilder.setReaderProtocol(new MyIReaderProtocol());
        //将新的修改后的参配设置给连接管理器
        manager.option(okOptionsBuilder.build());

        manager.registerReceiver(new SocketActionAdapter() {
            @Override
            public void onSocketConnectionSuccess(ConnectionInfo info, String action) {
                super.onSocketConnectionSuccess(info, action);
                logger.info("connect success");

            }

            @Override
            public void onSocketDisconnection(ConnectionInfo info, String action, Exception e) {
                super.onSocketDisconnection(info, action, e);
                logger.info("disconnect success");
            }

            @Override
            public void onSocketReadResponse(ConnectionInfo info, String action, OriginalData data) {
                super.onSocketReadResponse(info, action, data);
                messageListener.onMessageListener(data);
            }
        });
        return authInfo;
    }

}
